import _$package from "commit-spell/package.json";
const {
  name: pkgName
} = _$package;
import _$fs from 'fs';
const {
  promises: fs
} = _$fs;
import _$os from 'os';
const {
  homedir
} = _$os;
import yargs from 'yargs/yargs';
import debugFactory from 'debug';
import spellcheck from 'spellchecker';
import execa from 'execa';
import textExtensions from 'text-extensions';
const debug = debugFactory(`${pkgName}:parse`);
const {
  readFile
} = fs;
const homeDir = homedir();
debug(`homeDir: ${homeDir}`);
export const GIT_COMMITMSG_PATH = './.git/COMMIT_EDITMSG';
export const LOCAL_SPELLCHECKIGNORE_PATH = './.spellcheckignore';
export const LOCAL_VSCODE_SETTINGS_PATH = './.vscode/settings.json'; // ? Home directory is prepended to the following

export const GLOBAL_SPELLCHECKIGNORE_PATH = `${homeDir}/.config/_spellcheckignore`;
export const GLOBAL_VSCODE_SETTINGS_PATH = `${homeDir}/.config/Code/User/settings.json`;

const tryReadFile = async path => {
  try {
    debug(`attempting to read ${path}`);
    const data = await readFile(path);
    debug(`successfully read ${path}`);
    return data.toString('utf-8');
  } catch (ignored) {
    debug.extend('<warn>')(`failed to read ${path}`);
    return '';
  }
};

const asCSpellJson = str => {
  try {
    const json = JSON.parse(str);
    debug('json parse successful!');
    return [...((json === null || json === void 0 ? void 0 : json['cSpell.words']) || []), ...((json === null || json === void 0 ? void 0 : json['cSpell.userWords']) || []), ...((json === null || json === void 0 ? void 0 : json['cSpell.ignoreWords']) || [])];
  } catch (ignored) {
    debug.extend('<warn>')('json parse failed');
    return [];
  }
};

const asText = item => item.split('\n');

const isPascalCase = word => /^([A-Z]{2,}.+|[A-Z][a-z]+[A-Z].*)$/.test(word);

const isCamelCase = word => /^[a-z]+[A-Z]+.*$/.test(word);

const isAllCaps = word => /^[^a-z]+$/.test(word);

const splitOutWords = phrase => {
  return [...phrase.split(/[^a-zA-Z]+/g), phrase].filter(Boolean);
};

const keys = obj => Object.keys(obj || {}).map(splitOutWords);
/**
 * Create and return a pre-configured Yargs instance (program) and argv parser.
 */


export function configureProgram(program) {
  const finalProgram = program || yargs();
  finalProgram.scriptName('commit-spell').usage(`$0\n\nChecks the ${GIT_COMMITMSG_PATH} file for spelling errors.`).options({
    silent: {
      alias: 's',
      describe: 'Nothing will be printed to stdout or stderr',
      type: 'boolean'
    },
    'show-all': {
      alias: 'a',
      describe: 'Show all potential misspellings',
      type: 'boolean'
    }
  }).string('_').group(['show-all'], 'Spellchecking options:').group(['help', 'version', 'silent'], 'Other options:').example([['$0', 'Stages file1 & commits as "feat: new feature"'], ['$0 --show-all', 'Stages file1 & file2 & commits as "feat(file1): new feature"']]).strictOptions();
  return {
    program: finalProgram,
    parse: async argv => {
      var _argv;

      argv = (_argv = argv) !== null && _argv !== void 0 && _argv.length ? argv : process.argv.slice(2);
      debug('saw argv: %O', argv);
      const finalArgv = await finalProgram.parse(argv || []);
      const lastCommitMsg = await tryReadFile(GIT_COMMITMSG_PATH);
      /* eslint-disable import/no-unresolved */

      const {
        dependencies: pkgDependencies,
        devDependencies: pkgDevDependencies,
        peerDependencies: pkgPeerDependencies,
        scripts: pkgScripts // @ts-expect-error: <cwd>/package.json might not exist

      } = await import('./package.json').catch(() => ({
        dependencies: {},
        devDependencies: {},
        peerDependencies: {},
        scripts: {}
      }));
      /* eslint-enable import/no-unresolved */

      debug(`lastCommitMsg: ${lastCommitMsg}`);
      const ignoreWords = Array.from(new Set([...(await Promise.all([tryReadFile(LOCAL_SPELLCHECKIGNORE_PATH).then(asText), tryReadFile(GLOBAL_SPELLCHECKIGNORE_PATH).then(asText), tryReadFile(LOCAL_VSCODE_SETTINGS_PATH).then(asCSpellJson), tryReadFile(GLOBAL_VSCODE_SETTINGS_PATH).then(asCSpellJson)])), ...textExtensions, // ? Popular contractions
      ...['ve', 're', 's', 'll', 't', 'd', 'o', 'ol'], ...keys(pkgDependencies), ...keys(pkgDevDependencies), ...keys(pkgPeerDependencies), ...keys(pkgScripts), ...splitOutWords((await execa('git', ['log', '--format="%B"', 'HEAD~1'])).stdout).slice(0, -1)].flat().filter(Boolean).flatMap(word => splitOutWords(word.trim().toLowerCase()))));
      const typos = Array.from(new Set(spellcheck.checkSpelling(lastCommitMsg).map(typoLocation => lastCommitMsg.slice(typoLocation.start, typoLocation.end).trim().split("'")).flat().filter(word => !isAllCaps(word) && !isCamelCase(word) && !isPascalCase(word)).map(word => word.toLowerCase()).filter(typo => !ignoreWords.includes(typo))));
      debug('typos: %O', typos);

      if (typos.length) {
        const warn = (...args) => {
          // eslint-disable-next-line no-console
          if (!finalArgv.silent) console.warn(...args);
        };

        warn('WARNING: there may be misspelled words in your commit message!');
        warn('Commit messages can be fixed before push with `git commit -S --amend`');
        warn('---');

        for (const typo of finalArgv.showAll ? typos : typos.slice(0, 5)) {
          const corrections = spellcheck.getCorrectionsForMisspelling(typo);
          const suggestion = corrections.length ? ` (did you mean ${corrections.slice(0, finalArgv.showAll ? 10 : 5).join(', ')}?)` : '';
          warn(`${typo}${suggestion}`);
        }

        if (typos.length) {
          if (!finalArgv.showAll && typos.length > 5) {
            warn(`${typos.length - 5} more...`);
          }

          warn('---');
        }
      }

      return finalArgv;
    }
  };
}